@@ -192,7 +192,7 @@ void stats_read(const char *stats_file, unsigned counters[STATS_END])
 {
 	int fd;
 
-	fd = open(stats_file, O_RDONLY);
+	fd = open(stats_file, O_RDONLY|O_BINARY);
 	if (fd == -1) {
 		stats_default(counters);
 		return;
@@ -230,6 +230,8 @@ void stats_summary(void)
 
 	}
 
+	printf("cache directory                     %s\n", cache_dir);
+
 	/* and display them */
 	for (i=0;stats_info[i].message;i++) {
 		enum stats stat = stats_info[i].stat;
