@@ -221,15 +221,18 @@ basetests() {
     checkstat 'cache hit' 10
     checkstat 'cache miss' 38
 
-    testname="direct .ii file"
-    mv test1.i test1.ii
-    $CCACHE_COMPILE -c test1.ii
-    checkstat 'cache hit' 10
-    checkstat 'cache miss' 39
+    # removed these tests as some compilers (including newer versions of gcc)
+    # determine which language to use based on .ii/.i extension, and C++ may 
+    # not be installed
+#     testname="direct .ii file"
+#     mv test1.i test1.ii
+#     $CCACHE_COMPILE -c test1.ii
+#     checkstat 'cache hit' 10
+#     checkstat 'cache miss' 39
 
-    $CCACHE_COMPILE -c test1.ii
-    checkstat 'cache hit' 11
-    checkstat 'cache miss' 39
+#     $CCACHE_COMPILE -c test1.ii
+#     checkstat 'cache hit' 11
+#     checkstat 'cache miss' 39
     
     testname="zero-stats"
     $CCACHE -z > /dev/null
